# -*- coding: utf-8 -*-

import os, sys
# ensure citruleaf.py is in path
sys.path.insert(1, os.path.realpath( os.pardir ) )

import unittest
import citrusleaf as cl

class TestQueryUsers(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        
        cls.cluster_with_user_admin = cl.CitrusleafCluster()
        cls.cluster_with_user_admin.setUser('testuauser', cl.hashpassword('aerospike'))
        cl.cluster_setHost(cls.cluster_with_user_admin, 'localhost', '3000')

        cls.cluster_with_read_user = cl.CitrusleafCluster()
        cls.cluster_with_read_user.setUser('testreaduser', cl.hashpassword('aerospike'))
        cl.cluster_setHost(cls.cluster_with_read_user, 'localhost', '3000')
    
    @classmethod
    def tearDownClass(cls):

        del(cls.cluster_with_user_admin)
        del(cls.cluster_with_read_user)

    def test_queryUsers(self):

        users = cl.queryUsers(TestQueryUsers.cluster_with_user_admin)

        self.assertEqual(0, users[0])
        self.assertNotEqual(0, len(users[1]))

    def test_queryUsers_with_other_than_useradmin_user(self):

        users = cl.queryUsers(TestQueryUsers.cluster_with_read_user)

        self.assertEqual(81, users[0])
        self.assertEqual([], users[1])

    def test_queryUsers_for_users_with_more_than_one_role(self):

        status = cl.grantRoles(TestQueryUsers.cluster_with_user_admin, "testreaduser", roles=["read-write", "sys-admin"])

        self.assertEqual(0, status)

        status = cl.grantRoles(TestQueryUsers.cluster_with_user_admin, "testsauser", roles=["read-write"])

        self.assertEqual(0, status)

        users = cl.queryUsers(TestQueryUsers.cluster_with_user_admin)

        self.assertEqual(0, users[0])

        for user in users[1]:

            if user.name == "testreaduser":
                self.assertEqual(["read", "read-write", "sys-admin"].sort(), user.roles.sort())
            if user.name == "testsauser":
                self.assertEqual(["read-write", "sys.admin"].sort(), user.roles.sort())

        status = cl.replaceRoles(TestQueryUsers.cluster_with_user_admin, "testreaduser", roles=["read"])
        status = cl.replaceRoles(TestQueryUsers.cluster_with_user_admin, "testsauser", roles=["sys-admin"])

    def test_queryUsers_for_users_with_more_than_one_role_with_useradmin_user(self):

        status = cl.grantRoles(TestQueryUsers.cluster_with_user_admin, "testuauser", roles=["read-write", "sys-admin"])

        self.assertEqual(0, status)

        users = cl.queryUsers(TestQueryUsers.cluster_with_user_admin)

        self.assertEqual(0, users[0])

        for user in users[1]:

            if user.name == "testuauser":
                self.assertEqual(["user-admin", "read-write", "sys-admin"].sort(), user.roles.sort())

        status = cl.replaceRoles(TestQueryUsers.cluster_with_user_admin, "testuauser", roles=["user-admin"])

if __name__ == "__main__":
    unittest.main()
